@@ -16,9 +16,9 @@
 from tempest import config
 from tempest.lib.common import ssh
 from tempest.lib import exceptions as lib_exc
-from tempest.scenario import manager
 
 from neutron_fwaas.tests.tempest_plugin.tests import fwaas_client
+from neutron_fwaas.tests.tempest_plugin.tests.scenario import manager
 
 CONF = config.CONF
 
